﻿using System;
using System.Collections.Generic;
using System.Linq;
using InSimDotNet.Packets;

namespace InSimDotNet {
    internal sealed class BindingManager {
        private readonly Dictionary<PacketType, IPacketBinding> bindings = new Dictionary<PacketType, IPacketBinding>();

        public void Bind(PacketType type, Action<InSim, IPacket> callback) {
            Bind<IPacket>(type, callback);
        }

        public void Bind<T>(PacketType type, Action<InSim, T> callback) where T : IPacket {
            IPacketBinding binding;
            if (bindings.TryGetValue(type, out binding)) {
                ((PacketBinding<T>)binding).Bind(callback);
            }
            else {
                bindings.Add(type, new PacketBinding<T>(callback));
            }
        }

        public void Unbind(PacketType type, Action<InSim, IPacket> callback) {
            Unbind<IPacket>(type, callback);
        }

        public void Unbind<T>(PacketType type, Action<InSim, T> callback) where T : IPacket {
            IPacketBinding binding;
            if (bindings.TryGetValue(type, out binding)) {
                var packetBinding = ((PacketBinding<T>)binding);
                packetBinding.Unbind(callback);

                if (!packetBinding.HasBindings) {
                    this.bindings.Remove(type);
                }
            }
        }

        public bool IsBound(PacketType type) {
            return bindings.ContainsKey(type);
        }

        public bool IsBound(PacketType type, Action<InSim, IPacket> callback) {
            return IsBound<IPacket>(type, callback);
        }

        public bool IsBound<T>(PacketType type, Action<InSim, T> callback) where T : IPacket {
            IPacketBinding binding;
            if (bindings.TryGetValue(type, out binding)) {
                return ((PacketBinding<T>)binding).IsBound(callback);
            }
            return false;
        }

        public void Execute(InSim insim, IPacket packet) {
            IPacketBinding binding;
            if (bindings.TryGetValue(packet.Type, out binding)) {
                binding.Execute(insim, packet);
            }
        }

        private interface IPacketBinding {
            void Execute(InSim insim, IPacket packet);
        }

        private sealed class PacketBinding<T> : IPacketBinding where T : IPacket {
            private List<WeakReference> references = new List<WeakReference>();

            public bool HasBindings {
                get { return references.Any(); }
            }

            public PacketBinding(Action<InSim, T> callback) {
                Bind(callback);
            }

            private static int GetBindingHashCode(Action<InSim, T> callback) {
                return new WeakReference(callback).Target.GetHashCode();
            }

            public void Bind(Action<InSim, T> callback) {
                references.Add(new WeakReference(callback));
            }

            public void Unbind(Action<InSim, T> callback) {
                var hashCode = GetBindingHashCode(callback);
                var reference = references.SingleOrDefault(r => r.Target.GetHashCode() == hashCode);
                if (reference != null) {
                    references.Remove(reference);
                }
            }

            public bool IsBound(Action<InSim, T> callback) {
                var hashCode = GetBindingHashCode(callback);
                return references.Any(r => r.Target.GetHashCode() == hashCode);
            }

            public void Execute(InSim insim, IPacket packet) {
                var temp = (T)packet;
                foreach (var reference in references) {
                    ((Action<InSim, T>)reference.Target)(insim, temp);
                }
            }
        }
    }
}
